--
-- Reversing Indoord Camera for FS19
-- You can look back throw the window when you are reversing with e.g. truck and trailer.
-- @author:    	kenny456 (kenny456@seznam.cz)
-- @history:	v1.0 - 2017-09-11 -
--				v1.1 - 2017-09-17 - added controls for position of reverse camera
--				v1.2 - 2017-09-27 - added option to turn on right camera
--				v1.3 - 2019-02-22 - converted to FS19
--								  - added option to disable reversing camera function
--								  - added MP support for set camera position
-- 
ReversingCamera = {};
ReversingCamera.modDirectory  = g_currentModDirectory
ReversingCamera.confDir = getUserProfileAppPath().. "modsSettings/ReversingCamera/";

function ReversingCamera.prerequisitesPresent(specializations)
	return true
end

function ReversingCamera.initSpecialization()
end

function ReversingCamera.registerOverwrittenFunctions(vehicleType)
end

function ReversingCamera.registerFunctions(vehicleType)
end

function ReversingCamera.registerEvents(vehicleType)
end

function ReversingCamera:registerActionEventsPlayer()
end

function ReversingCamera:registerActionEventsMenu()
end

function ReversingCamera:onRegisterActionEvents(isSelected, isOnActiveVehicle)
	if g_dedicatedServerInfo ~= nil then
		return
	end
	local spec = self.spec_ReversingCamera
	if isOnActiveVehicle and self:getIsControlled() then
		local actions_RC1 = { InputAction.RC_RESET_POSITION, InputAction.RC_TOGGLE_HELP, InputAction.RC_TOGGLE_RIGHT, InputAction.RC_CENTER_CAMERA, InputAction.RC_TOGGLE, InputAction.RC_TOGGLE_ALL }
		local actions_RC2 = { InputAction.RC_MOVE_OUT, InputAction.RC_MOVE_IN, InputAction.RC_MOVE_FORWARD, InputAction.RC_MOVE_BACK }

		for _,actionName in pairs(actions_RC1) do
			local _, eventID = g_inputBinding:registerActionEvent(actionName, self, ReversingCamera.actionCallback, true, true, false, spec.reverseCamera or spec.reverseCameraR)
			spec.event_IDs[actionName] = eventID
			if g_inputBinding ~= nil and g_inputBinding.events ~= nil and g_inputBinding.events[eventID] ~= nil then
				g_inputBinding.events[eventID].displayPriority = 2
				if actionName == InputAction.RC_TOGGLE_HELP then
					g_inputBinding:setActionEventTextVisibility(eventID, true)
				elseif actionName == InputAction.RC_TOGGLE or actionName == InputAction.RC_TOGGLE_ALL then
					g_inputBinding:setActionEventTextVisibility(eventID, false)
				else
					g_inputBinding:setActionEventTextVisibility(eventID, ReversingCamera.showHelp)
				end
			end
		end

		for _,actionName in pairs(actions_RC2) do
			local _, eventID = g_inputBinding:registerActionEvent(actionName, self, ReversingCamera.actionCallback, true, true, true, spec.reverseCamera or spec.reverseCameraR)	
			spec.event_IDs[actionName] = eventID
			if g_inputBinding ~= nil and g_inputBinding.events ~= nil and g_inputBinding.events[eventID] ~= nil then
				g_inputBinding.events[eventID].displayPriority = 1
				g_inputBinding:setActionEventTextVisibility(eventID, ReversingCamera.showHelp)
			end
		end
	end
end

function ReversingCamera.registerEventListeners(vehicleType)
	for _,n in pairs( { "onLoad", "onPostLoad", "saveToXMLFile", "onUpdate", "onUpdateTick", "onDraw", "onRegisterActionEvents", "registerActionEventsPlayer", "onEnterVehicle", "saveToXml", "loadFromXml", "onReadStream", "onWriteStream" } ) do
		SpecializationUtil.registerEventListener(vehicleType, n, ReversingCamera)
	end
end

function ReversingCamera:onLoad(savegame)
	local spec = self.spec_ReversingCamera
	local xmlFile = self.xmlFile
	spec.event_IDs = {}
	spec.rcActive = true
	spec.rcCamIndex = nil
	ReversingCamera.rcActive = true
	ReversingCamera.showHelp = true
	self.activateCamera = ReversingCamera.activateCamera
	self.loadFromXml = ReversingCamera.loadFromXml
	self.saveToXml = ReversingCamera.saveToXml
	self.toggleCamera = ReversingCamera.toggleCamera
	
	if self.spec_enterable ~= nil then
		local i = 0
		local key = 'vehicle.enterable.cameras'
		while true do
			local cameraKey = key .. string.format(".camera(%d)", i)
			if not hasXMLProperty(xmlFile, cameraKey) then
				break
			end
			local inside = Utils.getNoNil(getXMLBool(xmlFile, cameraKey.."#isInside"),false);
			local rotatable = Utils.getNoNil(getXMLBool(xmlFile, cameraKey.."#rotatable"),false);
			local node = Utils.getNoNil(getXMLString(xmlFile, cameraKey.."#node"),nil);
			if inside and rotatable and node ~= nil then
				local name = getXMLString(xmlFile, cameraKey .. "#node")
				if self.i3dMappings[name] ~= nil then
					spec.internalCamera = I3DUtil.indexToObject(self.components, self.i3dMappings[name])
				else
					spec.internalCamera = I3DUtil.indexToObject(self.components, name)
				end
				spec.rcCamIndex = i + 1
				break
			end
			i = i + 1
		end
		if spec.internalCamera ~= nil then
			local p = getParent(spec.internalCamera)
			spec.cam = createTransformGroup("cam")
			link(p, spec.cam)
			link(spec.cam, spec.internalCamera)
			xt,yt,zt = getTranslation(spec.cam);
			spec.camPositionX = xt;  
			spec.camPositionY = yt; 
			spec.camPositionZ = zt;
			spec.tx=xt;
			spec.ty=yt;
			spec.tz=zt;
		else
			if self.configFileName ~= nil then
				print('Reversing Camera - No inside camera found for vehicle: '..self.configFileName)
			end
		end
	end
	
	spec.reverseCamera = false;
	spec.reverseCameraR = false;
	spec.revCamR = false;
	spec.transXmaxL = 0.6;
	spec.transZmaxL = 0.1;
	spec.rotYstartL = 4.2;
	spec.transXmaxR = 0.6;
	spec.transZmaxR = 0.1;
	spec.rotYstartR = 2.3;
	
	local configFile = ReversingCamera.confDir .. "ReversingCameraConfig.xml";
	if fileExists(configFile) then
		ReversingCamera.configXml = loadXMLFile("ReversingCamera_XML", configFile);
		self:loadFromXml(self:getFullName());
	else
		createFolder(getUserProfileAppPath().. "modsSettings/");
		createFolder(ReversingCamera.confDir);
		ReversingCamera.configXml = createXMLFile("ReversingCamera_XML", configFile, "ReversingCameraConfig");
		self:saveToXml();
		saveXMLFile(ReversingCamera.configXml);
	end
end

function ReversingCamera:loadFromXml(vehicle)
	local spec = self.spec_ReversingCamera
    if ReversingCamera.configXml == nil then
        return;
    end
	if getXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.showHelp") ~= nil then
		ReversingCamera.showHelp = getXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.showHelp")
	end;
	if getXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveGlobal") ~= nil then
		ReversingCamera.rcActive = getXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveGlobal")
	end
	if vehicle ~= nil then
		local key = "ReversingCameraConfig.rcActiveVehicles"
		local i = 0
		while true do
			local name = getXMLString(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveVehicles"..string.format(".vehicle(%d)", i).."#name");
			if name ~= nil then
				if name == vehicle then
					spec.rcActive = getXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveVehicles"..string.format(".vehicle(%d)", i));
					break
				end
			else
				break
			end
			i = i + 1
		end
	end
end

function ReversingCamera:saveToXml(vehicle, active)
	local spec = self.spec_ReversingCamera
    if ReversingCamera.configXml == nil then
        return;
    end
	setXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.showHelp", ReversingCamera.showHelp);
	setXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveGlobal", ReversingCamera.rcActive);
	if vehicle ~= nil and active ~= nil then
		local key = "ReversingCameraConfig.rcActiveVehicles"
		local i = 0
		while true do
			local name = getXMLString(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveVehicles"..string.format(".vehicle(%d)", i).."#name");
			if name ~= nil then
				if name == vehicle then
					setXMLString(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveVehicles"..string.format(".vehicle(%d)", i).."#name", vehicle);
					setXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveVehicles"..string.format(".vehicle(%d)", i), active);
					break
				end
			else
				setXMLString(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveVehicles"..string.format(".vehicle(%d)", i).."#name", vehicle);
				setXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveVehicles"..string.format(".vehicle(%d)", i), active);
				break
			end
			i = i + 1
			local name = key .. string.format(".vehicle(%d)", i)
			if not hasXMLProperty(ReversingCamera.configXml, name) then
				setXMLString(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveVehicles"..string.format(".vehicle(%d)", i).."#name", vehicle);
				setXMLBool(ReversingCamera.configXml, "ReversingCameraConfig.rcActiveVehicles"..string.format(".vehicle(%d)", i), active);
				break
			end
		end
	end
    saveXMLFile(ReversingCamera.configXml);
end

function ReversingCamera:onPostLoad(savegame)
	local spec = self.spec_ReversingCamera
	if savegame ~= nil then
		local xmlFile = savegame.xmlFile
		local key = savegame.key ..".ReversingCamera"
		spec.transXmaxL = Utils.getNoNil(getXMLFloat(xmlFile, key.."#transXmaxL"), spec.transXmaxL);
		spec.transZmaxL = Utils.getNoNil(getXMLFloat(xmlFile, key.."#transZmaxL"), spec.transZmaxL);
		spec.transXmaxR = Utils.getNoNil(getXMLFloat(xmlFile, key.."#transXmaxR"), spec.transXmaxR);
		spec.transZmaxR = Utils.getNoNil(getXMLFloat(xmlFile, key.."#transZmaxR"), spec.transZmaxR);
		spec.revCamR = Utils.getNoNil(getXMLBool(xmlFile, key.."#revCamR"), spec.revCamR);
		self:toggleCamera(spec.transXmaxL, spec.transZmaxL, spec.transXmaxR, spec.transZmaxR, spec.revCamR, true)
	end
end

function ReversingCamera:saveToXMLFile(xmlFile, key)
	local spec = self.spec_ReversingCamera
	setXMLFloat(xmlFile, key.."#transXmaxL", spec.transXmaxL)
	setXMLFloat(xmlFile, key.."#transZmaxL", spec.transZmaxL)
	setXMLFloat(xmlFile, key.."#transXmaxR", spec.transXmaxR)
	setXMLFloat(xmlFile, key.."#transZmaxR", spec.transZmaxR)
	setXMLBool(xmlFile, key.."#revCamR", spec.revCamR)
end

function ReversingCamera:onUpdate(dt)
	local spec = self.spec_ReversingCamera
	if self:getIsActive() and self:getIsControlled() then
		if self.spec_enterable ~= nil and self.spec_enterable.cameras[self.spec_enterable.camIndex].isInside == true and spec.internalCamera ~= nil and (spec.rcActive and ReversingCamera.rcActive) and self.spec_enterable.camIndex == spec.rcCamIndex then
			local math_pi_double  = math.pi*2
			if math_pi_double > self.spec_enterable.cameras[self.spec_enterable.camIndex].rotY and self.spec_enterable.cameras[self.spec_enterable.camIndex].rotY > spec.rotYstartL then
					spec.reverseCamera = true;
					spec.transXmax = spec.transXmaxL;
					spec.transZmax = spec.transZmaxL;
					spec.rozdilY = spec.spec_enterable.cameras[spec.spec_enterable.camIndex].rotY - spec.rotYstartL;
				elseif spec.rotYstartR > spec.spec_enterable.cameras[spec.spec_enterable.camIndex].rotY and spec.spec_enterable.cameras[spec.spec_enterable.camIndex].rotY > 0 then
					spec.reverseCameraR = true;
					spec.transXmax = spec.transXmaxR;
					spec.transZmax = spec.transZmaxR;
					spec.rozdilY = spec.spec_enterable.cameras[spec.spec_enterable.camIndex].rotY - spec.rotYstartR;
				else
					spec.reverseCamera = false;
					spec.reverseCameraR = false;
				end;
			if spec.reverseCamera == true or (spec.reverseCameraR == true and spec.revCamR == true) then
				spec.camPositionX = (spec.tx + (spec.rozdilY * 0.5 * spec.transXmax / 0.6));
				if spec.reverseCameraR == true then
					spec.camPositionZ = (spec.tz + (-1 * spec.rozdilY * 0.5 * spec.transZmax / 0.1));
				else
					spec.camPositionZ = (spec.tz + (spec.rozdilY * 0.5 * spec.transZmax / 0.1));
				end;
				if spec.reverseCameraR == true then
					if spec.camPositionX <= spec.tx - spec.transXmax then
						spec.camPositionX = spec.tx - spec.transXmax;
					end;
					if spec.spec_enterable.cameras[spec.spec_enterable.camIndex].rotY <= 0.1 then
						spec.spec_enterable.cameras[spec.spec_enterable.camIndex].rotY = 0.1;
					end;
				else
					if spec.camPositionX >= spec.tx + spec.transXmax then
						spec.camPositionX = spec.tx + spec.transXmax;
					end;
					if spec.spec_enterable.cameras[spec.spec_enterable.camIndex].rotY >= 6 then
						spec.spec_enterable.cameras[spec.spec_enterable.camIndex].rotY = 6;
					end;
				end;
				if spec.camPositionZ >= spec.tz + spec.transZmax then
					spec.camPositionZ = spec.tz + spec.transZmax;
				end;
				if spec.input_RC_MOVE_OUT then
					if spec.transXmax <= spec.tx + 2 then
						if spec.reverseCameraR == true then
							spec.camPositionX = spec.camPositionX - 0.01;
							spec.transXmax = spec.transXmax + 0.01;
						else
							spec.camPositionX = spec.camPositionX + 0.01;
							spec.transXmax = spec.transXmax + 0.01;
						end;
					end;
				end;
				if spec.input_RC_MOVE_IN then
					if spec.transXmax >= 0.1 then
						if spec.reverseCameraR == true then
							spec.camPositionX = spec.camPositionX + 0.01;
							spec.transXmax = spec.transXmax - 0.01;
						else
							spec.camPositionX = spec.camPositionX - 0.01;
							spec.transXmax = spec.transXmax - 0.01;
						end;
					end;
				end;
				if spec.input_RC_MOVE_FORWARD then
					if spec.reverseCameraR == true then
						if spec.transZmax <= 1.5 then
							spec.camPositionZ = spec.camPositionZ + 0.01;
							spec.transZmax = spec.transZmax + 0.01;
						end;
					else
						if spec.transZmax <= 1.5 then
							spec.camPositionZ = spec.camPositionZ + 0.01;
							spec.transZmax = spec.transZmax + 0.01;
						end;
					end;
				end;
				if spec.input_RC_MOVE_BACK then
					if spec.reverseCameraR == true then
						if spec.transZmax >= 0.01 then
							spec.camPositionZ = spec.camPositionZ - 0.01;
							spec.transZmax = spec.transZmax - 0.01;
						end
					else
						if spec.transZmax >= 0.01 then
							spec.camPositionZ = spec.camPositionZ - 0.01;
							spec.transZmax = spec.transZmax - 0.01;
						end;
					end;
				end;
				if spec.reverseCameraR == true then
					spec.transXmaxR = spec.transXmax;
					spec.transZmaxR = spec.transZmax;
				else
					spec.transXmaxL = spec.transXmax;
					spec.transZmaxL = spec.transZmax;
				end;
			else
				spec.camPositionX = spec.tx;
				spec.camPositionZ = spec.tz;
			end;
			if spec.internalCamera ~= nil and spec.spec_enterable.cameras[spec.spec_enterable.camIndex].isInside == true then
				setTranslation(spec.cam, spec.camPositionX, spec.camPositionY, spec.camPositionZ);
			end;
		end;
		if spec.event_IDs ~= nil and g_dedicatedServerInfo == nil then
			if spec.reverseCamera == true  and self:getActiveCamera().isInside and self.spec_enterable.camIndex == spec.rcCamIndex then
				for i,eventID in pairs(spec.event_IDs) do
					if i == InputAction.RC_TOGGLE_RIGHT then
						g_inputBinding:setActionEventActive(eventID, false)
					else
						g_inputBinding:setActionEventActive(eventID, true)
					end
				end
			elseif spec.reverseCameraR == true  and self:getActiveCamera().isInside and self.spec_enterable.camIndex == spec.rcCamIndex then
				for i,eventID in pairs(spec.event_IDs) do
					if spec.revCamR then
						g_inputBinding:setActionEventActive(eventID, true)
						g_inputBinding:setActionEventText(spec.event_IDs['RC_TOGGLE_RIGHT'], g_i18n:getText('RC_RIGHT_OFF'))
					else
						if i == InputAction.RC_TOGGLE_RIGHT or i == InputAction.RC_TOGGLE_HELP then
							g_inputBinding:setActionEventActive(eventID, true)
						else
							g_inputBinding:setActionEventActive(eventID, false)
						end
						g_inputBinding:setActionEventText(spec.event_IDs['RC_TOGGLE_RIGHT'], g_i18n:getText('RC_RIGHT_ON'))
					end
				end
			else
				for i,eventID in pairs(spec.event_IDs) do
					g_inputBinding:setActionEventActive(eventID, false)
				end
			end
			if self:getActiveCamera().isInside and self.spec_enterable.camIndex == spec.rcCamIndex then
				if spec.rcActive and ReversingCamera.rcActive then
					g_inputBinding:setActionEventActive(spec.event_IDs['RC_CENTER_CAMERA'], true)
				end
				g_inputBinding:setActionEventActive(spec.event_IDs['RC_TOGGLE'], true)
				g_inputBinding:setActionEventActive(spec.event_IDs['RC_TOGGLE_ALL'], true)
				if ReversingCamera.showHelp then
					g_inputBinding:setActionEventText(spec.event_IDs['RC_TOGGLE_HELP'], g_i18n:getText('RC_HELP_HIDE'))
				else
					g_inputBinding:setActionEventText(spec.event_IDs['RC_TOGGLE_HELP'], g_i18n:getText('RC_HELP_SHOW'))
				end
			end
		end
	end;
end
function ReversingCamera:onUpdateTick(dt)
end

function ReversingCamera:onDraw()
end

function ReversingCamera:actionCallback(actionName, keyStatus, arg4, arg5, arg6)
	local spec = self.spec_ReversingCamera
	if self:getIsActive() then
		if keyStatus > 0 then
			if actionName == 'RC_MOVE_OUT' then
				spec.input_RC_MOVE_OUT = true
			elseif actionName == 'RC_MOVE_IN' then
				spec.input_RC_MOVE_IN = true
			elseif actionName == 'RC_MOVE_FORWARD' then
				spec.input_RC_MOVE_FORWARD = true
			elseif actionName == 'RC_MOVE_BACK' then
				spec.input_RC_MOVE_BACK = true
			elseif actionName == 'RC_RESET_POSITION' then
				if spec.reverseCamera then
					spec.transXmaxL = 0.6;
					spec.transZmaxL = 0.1;
				else
					spec.transXmaxR = 0.6;
					spec.transZmaxR = 0.1;
				end
				self:toggleCamera(spec.transXmaxL, spec.transZmaxL, spec.transXmaxR, spec.transZmaxR, spec.revCamR)
			elseif actionName == 'RC_TOGGLE_HELP' then
				ReversingCamera.showHelp = not ReversingCamera.showHelp
				if (spec.event_IDs ~= nil) then
					for i,eventID in pairs(spec.event_IDs) do
						if i ~= InputAction.RC_TOGGLE_HELP and i ~= InputAction.RC_TOGGLE and i ~= InputAction.RC_TOGGLE_ALL then
							g_inputBinding:setActionEventTextVisibility(eventID, ReversingCamera.showHelp)
						end
					end
				end
				self:saveToXml();
			elseif actionName == 'RC_TOGGLE_RIGHT' then
				if spec.reverseCameraR == true then
					spec.revCamR = not spec.revCamR;
				end
				self:toggleCamera(spec.transXmaxL, spec.transZmaxL, spec.transXmaxR, spec.transZmaxR, spec.revCamR)
			elseif actionName == 'RC_CENTER_CAMERA' then
				self.spec_enterable.cameras[self.spec_enterable.camIndex].rotY = self.spec_enterable.cameras[self.spec_enterable.camIndex].origRotY
				self.spec_enterable.cameras[self.spec_enterable.camIndex].rotX = self.spec_enterable.cameras[self.spec_enterable.camIndex].origRotX
			elseif actionName == 'RC_TOGGLE' then
				if ReversingCamera.rcActive == false then
					g_currentMission:showBlinkingWarning("FIRST ACTIVATE REVERSING CAMERA GLOBAL (CTRL + ALT + R)")
				else
					local text = 'ACTIVATED'
					spec.rcActive = not spec.rcActive
					self:activateCamera(spec.rcActive, 'THIS')
					self:saveToXml(self:getFullName(), spec.rcActive);
				end
			elseif actionName == 'RC_TOGGLE_ALL' then
				ReversingCamera.rcActive = not ReversingCamera.rcActive
				self:activateCamera(ReversingCamera.rcActive, 'ALL')
				self:saveToXml();
			end
		elseif keyStatus == 0 then
			if actionName == 'RC_MOVE_OUT' then
				spec.input_RC_MOVE_OUT = false
				self:toggleCamera(spec.transXmaxL, spec.transZmaxL, spec.transXmaxR, spec.transZmaxR, spec.revCamR)
			elseif actionName == 'RC_MOVE_IN' then
				spec.input_RC_MOVE_IN = false
				self:toggleCamera(spec.transXmaxL, spec.transZmaxL, spec.transXmaxR, spec.transZmaxR, spec.revCamR)
			elseif actionName == 'RC_MOVE_FORWARD' then
				spec.input_RC_MOVE_FORWARD = false
				self:toggleCamera(spec.transXmaxL, spec.transZmaxL, spec.transXmaxR, spec.transZmaxR, spec.revCamR)
			elseif actionName == 'RC_MOVE_BACK' then
				spec.input_RC_MOVE_BACK = false
				self:toggleCamera(spec.transXmaxL, spec.transZmaxL, spec.transXmaxR, spec.transZmaxR, spec.revCamR)
			elseif actionName == 'RC_RESET_POSITION' then
				spec.input_RC_RESET_POSITION = false
			elseif actionName == 'RC_TOGGLE_HELP' then
				spec.input_RC_TOGGLE_HELP = false
			elseif actionName == 'RC_TOGGLE_RIGHT' then
				spec.input_RC_TOGGLE_RIGHT = false
			end
		end
	end
end

function ReversingCamera:activateCamera(status, mode)
	local spec = self.spec_ReversingCamera
	if mode == 'THIS' then
		mode = self:getFullName()
	end
	local text = 'ACTIVATED'
	if not status then
		text = 'DEACTIVATED'
		spec.reverseCamera = false
		spec.reverseCameraR = false
		if spec.internalCamera ~= nil then
			setTranslation(spec.cam, spec.tx, spec.ty, spec.tz);
		end
	end
	g_currentMission:showBlinkingWarning("REVERSING CAMERA FUNCTIONS "..tostring(text).." FOR "..tostring(mode).." VEHICLES", 2000);
end

function ReversingCamera:onEnterVehicle()
	local spec = self.spec_ReversingCamera
	if spec.rcActive == false or ReversingCamera.rcActive == false then
		spec.reverseCamera = false
		spec.reverseCameraR = false
		if spec.internalCamera ~= nil then
			setTranslation(spec.cam, spec.tx, spec.ty, spec.tz);
		end
	end
	if (spec.event_IDs ~= nil) then
		for i,eventID in pairs(spec.event_IDs) do
			if i ~= InputAction.RC_TOGGLE_HELP and i ~= InputAction.RC_TOGGLE and i ~= InputAction.RC_TOGGLE_ALL then
				g_inputBinding:setActionEventTextVisibility(eventID, ReversingCamera.showHelp)
			end
		end
	end
	self:loadFromXml(self:getFullName());
end

function ReversingCamera:toggleCamera(transXmaxL, transZmaxL, transXmaxR, transZmaxR, revCamR, noEventSend)
	local spec = self.spec_ReversingCamera

	ReversingCameraToggleEvent.sendEvent(self, transXmaxL, transZmaxL, transXmaxR, transZmaxR,  revCamR, noEventSend)

	spec.transXmaxL = transXmaxL;
	spec.transZmaxL = transZmaxL;
	spec.transXmaxR = transXmaxR;
	spec.transZmaxR = transZmaxR;
	spec.revCamR = revCamR;
end

function ReversingCamera:onReadStream(streamId, connection)
	local spec = self.spec_ReversingCamera
	local transXmaxL = streamReadFloat32(streamId);
	local transZmaxL = streamReadFloat32(streamId);
	local transXmaxR = streamReadFloat32(streamId);
	local transZmaxR = streamReadFloat32(streamId);
    local revCamR = streamReadBool(streamId);
	if transXmaxL ~= nil or transZmaxL ~= nil or transXmaxR ~= nil or transZmaxR ~= nil or revCamR ~= nil then
		self:toggleCamera(transXmaxL, transZmaxL, transXmaxR, transZmaxR, revCamR, true)
	end
end;

function ReversingCamera:onWriteStream(streamId, connection)
	local spec = self.spec_ReversingCamera
	streamWriteFloat32(streamId, spec.transXmaxL);
	streamWriteFloat32(streamId, spec.transZmaxL);
	streamWriteFloat32(streamId, spec.transXmaxR);
	streamWriteFloat32(streamId, spec.transZmaxR);
	streamWriteBool(streamId, spec.revCamR);
end;

ReversingCameraToggleEvent = {};
ReversingCameraToggleEvent_mt = Class(ReversingCameraToggleEvent, Event);

InitEventClass(ReversingCameraToggleEvent, "ReversingCameraToggleEvent");

function ReversingCameraToggleEvent:emptyNew()
    local self = Event:new(ReversingCameraToggleEvent_mt);
    self.className="ReversingCameraToggleEvent";
    return self;
end;

function ReversingCameraToggleEvent:new(object, transXmaxL, transZmaxL, transXmaxR, transZmaxR, revCamR)
	local self = ReversingCameraToggleEvent:emptyNew()
	self.object = object;
	self.transXmaxL = transXmaxL;
	self.transZmaxL = transZmaxL;
	self.transXmaxR = transXmaxR;
	self.transZmaxR = transZmaxR;
	self.revCamR = revCamR;
	return self;
end;

function ReversingCameraToggleEvent:readStream(streamId, connection)
	self.object = NetworkUtil.readNodeObject(streamId);
	self.transXmaxL = streamReadFloat32(streamId);
	self.transZmaxL = streamReadFloat32(streamId);
	self.transXmaxR = streamReadFloat32(streamId);
	self.transZmaxR = streamReadFloat32(streamId);
    self.revCamR = streamReadBool(streamId);
    self:run(connection);
end;

function ReversingCameraToggleEvent:writeStream(streamId, connection)
	NetworkUtil.writeNodeObject(streamId, self.object);
	streamWriteFloat32(streamId, self.transXmaxL);
	streamWriteFloat32(streamId, self.transZmaxL);
	streamWriteFloat32(streamId, self.transXmaxR);
	streamWriteFloat32(streamId, self.transZmaxR);
	streamWriteBool(streamId, self.revCamR);
end;

function ReversingCameraToggleEvent:run(connection)
	if self.object ~= nil then
		self.object:toggleCamera(self.transXmaxL, self.transZmaxL, self.transXmaxR, self.transZmaxR, self.revCamR, true);
	end
	if not connection:getIsServer() then
		g_server:broadcastEvent(ReversingCameraToggleEvent:new(self.object, self.transXmaxL, self.transZmaxL, self.transXmaxR, self.transZmaxR, self.revCamR), nil, connection, self.object);
	end;
end;

function ReversingCameraToggleEvent.sendEvent(vehicle, transXmaxL, transZmaxL, transXmaxR, transZmaxR, revCamR, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(ReversingCameraToggleEvent:new(vehicle, transXmaxL, transZmaxL, transXmaxR, transZmaxR,  revCamR), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(ReversingCameraToggleEvent:new(vehicle, transXmaxL, transZmaxL, transXmaxR, transZmaxR, revCamR));
		end;
	end;
end;